/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */

#include <beattracker/voice_cancel.h>

VoiceCancel::VoiceCancel(size_t window_size) :
    window_size_(window_size) {
    //バッファー初期化
    s_x = (double **) calloc2dim(sizeof(double), window_size_ / 2 + 1, 2);
    s_y = (double **) calloc2dim(sizeof(double), window_size_ / 2 + 1, 2);
    s_d = (double **) calloc2dim(sizeof(double), window_size_ / 2 + 1, 2);
    semiblind_ica_ = new_SemiBlind(NULL, window_size_, 8, 5, 1, 1.0, 1.0, 0.01,
                                   0.01, 0.01, 0.00, 1.0e-4);
}

VoiceCancel::~VoiceCancel() {
    //メモリー解放
    delete_SemiBlind(semiblind_ica_, NULL);
    free(s_x);
    s_x = NULL;
    free(s_y);
    s_y = NULL;
    free(s_d);
    s_d = NULL;
}

int VoiceCancel::ExecuteVoiceCancel(
    const std::vector<FftComplex> self_voice_spec,
    const std::vector<FftComplex> mic_spec,
    std::vector<double> &voice_canceled_spec) {
    if (self_voice_spec.size() != (window_size_ / 2 + 1)
            || mic_spec.size() != (window_size_ / 2 + 1)
            || voice_canceled_spec.size() != (window_size_ / 2 + 1)) {
        return -1;
    } else {
        // 変数の初期化
        // set s_y
        for (size_t f = 0; f < window_size_ / 2 + 1; f++) {
            s_y[f][0] = mic_spec[f][0];
            s_y[f][1] = mic_spec[f][1];
        }
        // set s_x
        for (size_t f = 0; f < window_size_ / 2 + 1; f++) {
            s_x[f][0] = self_voice_spec[f][0];
            s_x[f][1] = self_voice_spec[f][1];
        }

        // エコーキャンセル実行
        separate_SemiBlind(semiblind_ica_, s_y, s_d, s_x, 1);

        //　出力
        for (size_t f = 0; f < window_size_ / 2 + 1; f++) {
            voice_canceled_spec[f] = sqrt(s_d[f][0] * s_d[f][0] + s_d[f][1] * s_d[f][1]);
        }
        return 0;
    }
}
